Index: include/parrot/interpreter.h
===================================================================
RCS file: /cvs/public/parrot/include/parrot/interpreter.h,v
retrieving revision 1.64
diff -u -r1.64 interpreter.h
--- include/parrot/interpreter.h	22 Jan 2003 15:57:03 -0000	1.64
+++ include/parrot/interpreter.h	3 May 2003 10:47:21 -0000
@@ -163,6 +163,9 @@
                                    requests are there? */
     UINTVAL GC_block_level;     /* How many outstanding GC block
                                    requests are there? */
+    int dod_skip_control;       /* Control skipping DOD runs 
+                                   0=no skip, 1=skip next, 2=last was skipped */
+    int last_total_free;        /* Number of free objects after last DOD run */
 
     PDB_t *pdb;                 /* Debug system */
     void *lo_var_ptr;           /* Pointer to memory on runops system stack */
Index: dod.c
===================================================================
RCS file: /cvs/public/parrot/dod.c,v
retrieving revision 1.49
diff -u -r1.49 dod.c
--- dod.c	21 Mar 2003 08:40:08 -0000	1.49
+++ dod.c	3 May 2003 10:47:22 -0000
@@ -484,16 +484,13 @@
 {
     struct Small_Object_Pool *header_pool;
     int j;
-    /* XXX these should go into the interpreter */
-    static int skip = 0;
-    static int last_total_free = 0;
     int total_free = 0;
 
     if (interpreter->DOD_block_level) {
         return;
     }
-    if (skip) {
-        skip = 0;
+    if (interpreter->dod_skip_control == 1) {
+        interpreter->dod_skip_control = 2;
         return;
     }
     Parrot_block_DOD(interpreter);
@@ -541,9 +538,11 @@
     /* Note it */
     interpreter->dod_runs++;
     /* if we don't have more free objects then last, skip next DOD run */
-    if (total_free < last_total_free)
-        skip = 1;
-    last_total_free = total_free;
+    if (total_free < interpreter->last_total_free)
+        interpreter->dod_skip_control = 1;
+    else
+        interpreter->dod_skip_control = 0;
+    interpreter->last_total_free = total_free;
     Parrot_unblock_DOD(interpreter);
     return;
 }
Index: smallobject.c
===================================================================
RCS file: /cvs/public/parrot/smallobject.c,v
retrieving revision 1.21
diff -u -r1.21 smallobject.c
--- smallobject.c	13 Jan 2003 17:04:48 -0000	1.21
+++ smallobject.c	3 May 2003 10:47:24 -0000
@@ -138,23 +138,25 @@
     }
     pool->total_objects += pool->objects_per_alloc;
 
-    /* Allocate more next time */
-    if (GC_DEBUG(interpreter)) {
-        pool->objects_per_alloc *= GC_DEBUG_UNITS_PER_ALLOC_GROWTH_FACTOR;
-        pool->replenish_level =
-                (size_t)(pool->total_objects *
-                GC_DEBUG_REPLENISH_LEVEL_FACTOR);
-    }
-    else {
-        pool->objects_per_alloc *= UNITS_PER_ALLOC_GROWTH_FACTOR;
-        pool->replenish_level =
-                (size_t)(pool->total_objects * REPLENISH_LEVEL_FACTOR);
-    }
-    /* check alloc size against maximum */
-    size = pool->object_size * pool->objects_per_alloc;
+    /* Allocate more next time (unless last DOD run was skipped) */
+    if (interpreter->dod_skip_control != 2) {
+        if (GC_DEBUG(interpreter)) {
+            pool->objects_per_alloc *= GC_DEBUG_UNITS_PER_ALLOC_GROWTH_FACTOR;
+            pool->replenish_level =
+                    (size_t)(pool->total_objects *
+                    GC_DEBUG_REPLENISH_LEVEL_FACTOR);
+        }
+        else {
+            pool->objects_per_alloc *= UNITS_PER_ALLOC_GROWTH_FACTOR;
+            pool->replenish_level =
+                    (size_t)(pool->total_objects * REPLENISH_LEVEL_FACTOR);
+        }
+        /* check alloc size against maximum */
+        size = pool->object_size * pool->objects_per_alloc;
 
-    if (size > POOL_MAX_BYTES) {
-        pool->objects_per_alloc = POOL_MAX_BYTES / pool->object_size;
+        if (size > POOL_MAX_BYTES) {
+            pool->objects_per_alloc = POOL_MAX_BYTES / pool->object_size;
+        }
     }
 }
 
